@@ -3,7 +3,7 @@
  * @(#)main.c
  *
  * Copyright 1997-1999, Wes Cherry   (mailto:wesc@technosis.com)
- *           2000-2001, Aaron Ardiri (mailto:aaron@ardiri.com)
+ *           2000-2003, Aaron Ardiri (mailto:aaron@ardiri.com)
  * All rights reserved.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -35,52 +35,116 @@
  *                 Merged 68K and LE32 version into one binary
  *     12-Jan-2001 Renaud Malaval
  *                 Added 'wrdl' resource support
+ *      4-Jun-2002 Ben Combee
+ *                 Added more control over AUTOID numbers
+ *      3-Oct-2002 Ben Combee
+ *                 Changed include path mechanism to allow unlimited
+ *                 numbers of include paths (constrained by memory)
+ *     23-Oct-2003 Don't print copyright string when -q used
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef PALM_INTERNAL
+#include <io.h>
+#include <direct.h>
+#endif
 #include "pilrc.h"
+#include "version.h"
 #include "restype.h"
 
+#if !defined(__GNUC__) && !defined(__MWERKS__)
+#ifdef PALM_INTERNAL
+#pragma message( "- Palm internal version" )
+#else
+#pragma message( "- Standard version" )
+#endif
+#endif
+
+#define Str(x)  #x
+#define XStr(x) Str(x)
+
+
+/**
+ *  Display the PilRC Copyright (c) strings, but only once
+ */
+static void header(void)
+{
+	static int outputHeader = 0;
+	
+	if (outputHeader == 0)
+	{
+		outputHeader = 1;
+		printf
+		  ("PilRC v%s\n"
+		   "  Copyright 1997-1999 Wes Cherry   (wesc@ricochet.net)\n"
+		   "  Copyright 2000-2003 Aaron Ardiri (aaron@ardiri.com)\n"
+		   "\n", PILRC_VERSION_STR);
+	}
+}
+
 /**
  * Display the usage information for PilRC.
  */
-void
+static void
 Usage(void)
 {
-  printf
-    ("\nThis program is free software; you may redistribute it under the\n"
+  header();
+  Error
+    ("This program is free software; you may redistribute it under the\n"
      "terms of the GNU General Public License. This program has absolutely\n"
      "no warranty, you use it AS IS at your own risk.\n\n"
      "usage: pilrc {<options>} infile [outfiledir]\n\n" "Options:\n"
-     "        -L LANGUAGE  Compile resources for specific language\n"
-     "        -I <path>    Search for bitmap and include files in <path>\n"
-     "                     More than one -I <path> options may be given\n"
-     "                     The current directory is always searched\n"
-     "        -R <resfile> Generate JUMP/PilA .res file\n"
-     "        -ro          Generate resource database file instead of .bins\n"
-     "        -o <filedir> Equivalent to [outfiledir]\n"
-     "        -H <incfile> Autoassign IDs and write .h file with #defines\n"
-     "        -D <macro>   Define a pre-processor macro symbol\n"
-     "        -F5          Use Big5 Chinese font widths\n"
-     "        -Fkt         Use Korean font widths (hantip font)\n"
-     "        -Fkm         Use Korean font widths (hanme font)\n"
-     "        -Fg          Use GB Chinese font widths\n"
-     "        -Fj          Use Japense font widths\n"
-     "        -Fh          Use Hebrew font widths\n"
-     "        -rtl         Right to left support\n"
-     "        -q           Less noisy output\n"
-     "        -V           Generate M$ (VS-type) error/warning output\n"
-     "        -allowEditID Allow edit menu IDs (10000-10007)\n"
-     "        -PalmRez     Generate res with PalmRez option\n"
-     "        -LE32        Generate 32 bit little endian (ARM, NT) resources\n"
-     "        -Loc <code>  Compile only res with the attribute LOCALE \"code\"\n"
-     "                     code samples: deDE, esES, enUS, frFR, itIT, jpJP\n"
-     "        -StripLoc    Don't compile 'non localisable resources'\n"
-     "        <outfiledir> Directory where .bin files should be generated,\n"
-     "                     or name of the file to generate containing all\n"
-     "                     the generated resources\n");
+     "        -L LANGUAGE    Use the TRANSLATION section for the given language\n"
+     "                       Up to " XStr(MAXLANG) " -L options may be given\n"
+     "        -I <path>      Search for bitmap and include files in <path>\n"
+     "                       More than one -I <path> options may be given\n"
+     "                       The current directory is always searched\n"
+#ifdef PALM_INTERNAL
+     "        -noIFIH        Parse includes files in header files\n"
+#else
+     "        -noIFIH        Ignore includes files in header files\n"
+#endif
+     "        -R <resfile>   Generate JUMP/PilA .res file\n"
+     "        -ro            Generate resource database file instead of .bins\n"
+     "        -ts            put POSIX timestamp on .ro file generated\n"
+     "        -o <filedir>   Equivalent to [outfiledir]\n"
+     "        -H <incfile>   Autoassign IDs and write .h file with #defines\n"
+     "        -idStart <n>   Start autoID assignment at number <n>, counting up\n"
+     "        -D <macro>     Define a pre-processor macro symbol\n"
+     "        -F5            Use Big5 Chinese font widths\n"
+     "        -Fkt           Use Korean font widths (hantip font)\n"
+     "        -Fkm           Use Korean font widths (hanme font)\n"
+     "        -Fg            Use GB Chinese font widths\n"
+     "        -Fj            Use Japense font widths\n"
+     "        -Fh            Use Hebrew font widths\n"
+     "        -Fc            Use Cyrillic font widths\n"
+     "        -rtl           Right to left support\n"
+     "        -q             Less noisy output\n"
+     "        -V             Generate M$ (VS-type) error/warning output\n"
+     "        -allowEditID   Disable warnings about menu IDs in the Edit (10000-10007) range\n"
+     "        -allowBadSize  Disable warnings about icon sizes being incorrect\n"
+     "        -allowLargeRes Disable warnings about resources that exceed max HotSync resource size\n"
+     "        -noEllipsis    Disable special handling of \"...\" and ellipsis char\n"
+     "        -PalmRez       Generate res with PalmRez option\n"
+     "        -LE32          Generate 32 bit little endian (ARM, NT) resources\n"
+#ifdef PALM_INTERNAL
+     "        -AppIcon68K    Force AppIcon resources generation in 68K format\n"
+     "        -amdc <name>   Generate resource 'amdc' id=1 with <name>.dll as content\n"
+#endif
+     "        -Loc <code>    Compile only res with the attribute LOCALE \"code\"\n"
+     "                       code samples: deDE, esES, enUS, frFR, itIT, jpJP\n"
+     "        -StripLoc      Don't compile 'non localisable resources'\n"
+     "        -type          Specify the type to use when generating a prc (-ro)\n"
+     "        -creator       Specify the creator to use when generating a prc (-ro)\n"
+     "        -name          Specify the database name when generating a prc (-ro)\n"
+     "        <outfiledir>   Directory where .bin files should be generated,\n"
+     "                       or name of the file to generate containing all\n"
+     "                       the generated resources\n"
+     "        -M             Generate dependency list only; suppress all other output\n"
+     "        -MD            Generate dependency list"
+     );
   exit(1);
 }
 
@@ -101,14 +165,12 @@ main(int cArg,
   char *szMacro;
   char *szValue;
   char *szIncFile;
+  char *szIDStart;
   int i;
   int fontType;
   int macroValue;
 
-  // display the (c) string
-  printf("PilRC v2.8 patch pre-release 7\n");
-  printf("  Copyright 1997-1999 Wes Cherry   (wesc@ricochet.net)\n");
-  printf("  Copyright 2000-2001 Aaron Ardiri (aaron@ardiri.com)\n");
+  BOOL vfShowVersion = fFalse;
 
   // initialize
   if (cArg < 2)
@@ -125,6 +187,16 @@ main(int cArg,
   vfPrcType = NULL;
   vfStripNoLocRes = fFalse;
   szLocaleP = NULL;
+  szDllNameP = NULL;
+  //  vfIFIH = fFalse;
+#ifdef PALM_INTERNAL
+  vfIFIH = fTrue;                                // RMa Default no Parse include in header file
+#else
+  vfIFIH = fFalse;
+#endif
+  vfLE32 = fFalse;
+  vfAppIcon68K = fFalse;
+  vfAutoAmdc = fFalse;
 
   // process as many command line arguments as possible
   for (i = 1; i < cArg; i++)
@@ -150,10 +222,10 @@ main(int cArg,
     // language
     if (FSzEqI(rgszArg[i], "-L"))
     {
-      if (i++ == cArg)
+      if (i++ == cArg || totalLanguages >= MAXLANG)
         Usage();
 
-      szLanguage = rgszArg[i];
+      aszLanguage[totalLanguages++] = rgszArg[i];
       continue;
     }
 
@@ -163,12 +235,7 @@ main(int cArg,
       if (i++ == cArg)
         Usage();
 
-      if (totalIncludePaths == MAXPATHS)
-      {
-        printf("Too many include paths!\n\n");
-        Usage();
-      }
-      includePaths[totalIncludePaths++] = rgszArg[i];
+	  AddAccessPath(rgszArg[i]);
       continue;
     }
 
@@ -217,6 +284,19 @@ main(int cArg,
       continue;
     }
 
+    // ID value control -- Increasing IDs
+    if (FSzEqI(rgszArg[i], "-idStart"))
+    {
+      if (i++ == cArg)
+        Usage();
+        
+      szIDStart = rgszArg[i];
+      // reset ID start sequence to this number and start counting up.
+      vfAutoStartID = atoi(szIDStart);
+      idAutoDirection = +1;
+      continue;
+    }
+
     // be quiet?
     if (FSzEqI(rgszArg[i], "-q"))
     {
@@ -231,6 +311,27 @@ main(int cArg,
       continue;
     }
 
+    // allow "edit" ID's?
+    if (FSzEqI(rgszArg[i], "-allowBadSize"))
+    {
+      vfAllowBadIconSizes = fTrue;
+      continue;
+    }
+
+    // allow "edit" ID's?
+    if (FSzEqI(rgszArg[i], "-allowLargeRes"))
+    {
+      vfAllowLargeResources = fTrue;
+      continue;
+    }
+
+    // disable ellipsis processing?
+    if (FSzEqI(rgszArg[i], "-noEllipsis"))
+    {
+      vfNoEllipsis = fTrue;
+      continue;
+    }
+
     // No default flag for form and object form
     if (FSzEqI(rgszArg[i], "-PalmRez"))
     {
@@ -287,6 +388,13 @@ main(int cArg,
       continue;
     }
 
+    // font cyrillic
+    if (FSzEqI(rgszArg[i], "-Fc"))
+    {
+      fontType = fontCyrillic;
+      continue;
+    }
+
     // right to left?
     if (FSzEqI(rgszArg[i], "-rtl"))
     {
@@ -301,6 +409,24 @@ main(int cArg,
       continue;
     }
 
+#ifdef PALM_INTERNAL
+    // AppIcon68K
+    if (FSzEqI(rgszArg[i], "-AppIcon68K"))
+    {
+      vfAppIcon68K = fTrue;
+      continue;
+    }
+
+    if (FSzEqI(rgszArg[i], "-amdc"))
+    {
+      if (i++ == cArg)
+        Usage();
+      szDllNameP = rgszArg[i];
+      vfAutoAmdc = fTrue;
+      continue;
+    }
+#endif
+
     // Output a 'ro' File
     if (FSzEqI(rgszArg[i], "-ro"))
     {
@@ -308,6 +434,25 @@ main(int cArg,
       continue;
     }
 
+    // place timestamp (POSIX standard) on .ro file
+    if (FSzEqI(rgszArg[i], "-ts"))
+    {
+      vfPrcTimeStamp = fTrue;
+      continue;
+    }
+    /*
+     * LDu 31-8-2001 : Ignore Include File In Header Files
+     */
+    if (FSzEqI(rgszArg[i], "-noIFIH"))
+    {
+#ifdef PALM_INTERNAL
+      vfIFIH = fFalse;
+#else
+      vfIFIH = fTrue;
+#endif
+      continue;
+    }
+
     // Output filename
     if (FSzEqI(rgszArg[i], "-o"))
     {
@@ -350,12 +495,42 @@ main(int cArg,
       continue;
     }
 
+    // dependency tracking, no output files
+    if (FSzEqI(rgszArg[i], "-M"))
+    {
+        vfTrackDepends = fTrue;
+        vfInhibitOutput = fTrue;
+        continue;
+    }
+    
+    // dependency tracking, compiler resources too
+    if (FSzEqI(rgszArg[i], "-MD"))
+    {
+        vfTrackDepends = fTrue;
+        continue;
+    }
+
+    if (FSzEqI(rgszArg[i], "--version"))
+    {
+        vfShowVersion = fTrue;
+        continue;
+    }
+
     // unknown argument?
     Usage();
   }
+
+  // display the (c) string
+  if (!vfQuiet)
+  {
+    header();
+  }
+
+  if (vfShowVersion)
+    return 0;  // Stop prematurely, after showing the header
+
   if ((cArg - i) < 1)
     Usage();
-  printf("\n");
 
   if ((!szLocaleP) && (vfStripNoLocRes))
   {
@@ -372,18 +547,31 @@ main(int cArg,
   //  else
   //    szOutputPath = ".";
 
+#ifdef PALM_INTERNAL
+  // if output folder not exist create it
+  if (strstr(szOutputPath, ".prc") == NULL)
+    if ((_access(szOutputPath, 0)) == -1)
+    {
+      _mkdir(szOutputPath);
+    }
+#endif
+
   // last minute check? (extra stuff?)
   if (cArg != i)
     Usage();
 
-  if (vfLE32)
-    printf("Generating LE32 resources (ARM/NT) from '%s'.\n", szInputFile);
-  else
-    printf("Generating 68K resources from '%s'.\n", szInputFile);
+  if (!vfQuiet)
+  {
+    if (vfLE32)
+      printf("Generating LE32 resources (ARM/NT) from '%s'.\n", szInputFile);
+    else
+      printf("Generating 68K resources from '%s'.\n", szInputFile);
+  }
 
   ResTypeInit();
   CbInit();
-  ParseFile(szInputFile, szOutputPath, szResFile, szIncFile, fontType);
-
+  FreeRcpfile(ParseFile(szInputFile, szOutputPath, szResFile, szIncFile, fontType));
+  FreeAccessPathsList();
+  
   return 0;
 }
